"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const OracleDBHelper_1 = require("./OracleDBHelper");
const RESULT_VARIABLE_NAME = 'result';
class ProcedureFunctionHelper {
    constructor(db, callParameters) {
        this._bindParameters = {};
        this._db = db;
        if (!callParameters.procedureFunctionName) {
            throw new Error('Call parameters must have procedureFunctionName defined');
        }
        this._callParameters = callParameters;
    }
    async execute() {
        await this.prepareQuery();
        this.prepareBindParameters();
        const result = await this._db.execute(this._query, this._bindParameters);
        return (this._callParameters.isFunction) ? this._db.getDataFromCursorInResult(result, RESULT_VARIABLE_NAME) : undefined;
    }
    async prepareQuery() {
        await this.prepareQueryData();
        this._query = `begin
    ${this.prepareFunctionReusltVariable()}${this._callParameters.procedureFunctionName}(${this._parameters});
end;`;
    }
    prepareFunctionReusltVariable() {
        return (this._callParameters.isFunction) ? `:${RESULT_VARIABLE_NAME} := ` : '';
    }
    async prepareQueryData() {
        this.setProcedureParameters();
        await this.prepareObjectClasses();
    }
    setProcedureParameters() {
        const parameters = [];
        for (const item of this._callParameters.data) {
            parameters.push(`${item.name} => :${item.name}`);
        }
        this._parameters = parameters.join(',');
    }
    async prepareObjectClasses() {
        for (const item of this._callParameters.data) {
            if (item.isComplexType) {
                item.objectClass = await this._db.getDbObjectClass(item.type);
            }
        }
    }
    prepareBindParameters() {
        if (this._callParameters.isFunction) {
            OracleDBHelper_1.OracleDBHelper.prepareCursorOutputBindParameter();
            Object.assign(this._bindParameters, OracleDBHelper_1.OracleDBHelper.prepareCursorOutputBindParameter(RESULT_VARIABLE_NAME));
        }
        for (const item of this._callParameters.data) {
            this._bindParameters[item.name] = item.bindParameterValue;
        }
    }
}
exports.default = ProcedureFunctionHelper;
//# sourceMappingURL=ProcedureFunctionHelper.js.map